import { Construct } from 'constructs';
import { ActionCategory, ActionArtifactBounds } from './action';
import { Construct as CoreConstruct } from '../../core';
/**
 * (experimental) The creation attributes used for defining a configuration property of a custom Action.
 *
 * @experimental
 */
export interface CustomActionProperty {
    /**
     * (experimental) The name of the property.
     *
     * You use this name in the `configuration` attribute when defining your custom Action class.
     *
     * @experimental
     */
    readonly name: string;
    /**
     * (experimental) The description of the property.
     *
     * @default the description will be empty
     * @experimental
     */
    readonly description?: string;
    /**
     * (experimental) Whether this property is a key.
     *
     * @default false
     * @see https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-key
     * @experimental
     */
    readonly key?: boolean;
    /**
     * (experimental) Whether this property is queryable.
     *
     * Note that only a single property of a custom Action can be queryable.
     *
     * @default false
     * @see https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-codepipeline-customactiontype-configurationproperties.html#cfn-codepipeline-customactiontype-configurationproperties-queryable
     * @experimental
     */
    readonly queryable?: boolean;
    /**
     * (experimental) Whether this property is required.
     *
     * @experimental
     */
    readonly required: boolean;
    /**
     * (experimental) Whether this property is secret, like a password, or access key.
     *
     * @default false
     * @experimental
     */
    readonly secret?: boolean;
    /**
     * (experimental) The type of the property, like 'String', 'Number', or 'Boolean'.
     *
     * @default 'String'
     * @experimental
     */
    readonly type?: string;
}
/**
 * (experimental) Properties of registering a custom Action.
 *
 * @experimental
 */
export interface CustomActionRegistrationProps {
    /**
     * (experimental) The category of the Action.
     *
     * @experimental
     */
    readonly category: ActionCategory;
    /**
     * (experimental) The artifact bounds of the Action.
     *
     * @experimental
     */
    readonly artifactBounds: ActionArtifactBounds;
    /**
     * (experimental) The provider of the Action.
     *
     * For example, `'MyCustomActionProvider'`
     *
     * @experimental
     */
    readonly provider: string;
    /**
     * (experimental) The version of your Action.
     *
     * @default '1'
     * @experimental
     */
    readonly version?: string;
    /**
     * (experimental) The URL shown for the entire Action in the Pipeline UI.
     *
     * @default none
     * @experimental
     */
    readonly entityUrl?: string;
    /**
     * (experimental) The URL shown for a particular execution of an Action in the Pipeline UI.
     *
     * @default none
     * @experimental
     */
    readonly executionUrl?: string;
    /**
     * (experimental) The properties used for customizing the instance of your Action.
     *
     * @default []
     * @experimental
     */
    readonly actionProperties?: CustomActionProperty[];
}
/**
 * (experimental) The resource representing registering a custom Action with CodePipeline.
 *
 * For the Action to be usable, it has to be registered for every region and every account it's used in.
 * In addition to this class, you should most likely also provide your clients a class
 * representing your custom Action, extending the Action class,
 * and taking the `actionProperties` as properly typed, construction properties.
 *
 * @experimental
 */
export declare class CustomActionRegistration extends CoreConstruct {
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: CustomActionRegistrationProps);
}
