import { Construct } from 'constructs';
/**
 * (experimental) Configuration for Cognito sending emails via Amazon SES.
 *
 * @experimental
 */
export interface UserPoolSESOptions {
    /**
     * (experimental) The verified Amazon SES email address that Cognito should use to send emails.
     *
     * The email address used must be a verified email address
     * in Amazon SES and must be configured to allow Cognito to
     * send emails.
     *
     * @see https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-email.html
     * @experimental
     */
    readonly fromEmail: string;
    /**
     * (experimental) An optional name that should be used as the sender's name along with the email.
     *
     * @default - no name
     * @experimental
     */
    readonly fromName?: string;
    /**
     * (experimental) The destination to which the receiver of the email should reploy to.
     *
     * @default - same as the fromEmail
     * @experimental
     */
    readonly replyTo?: string;
    /**
     * (experimental) The name of a configuration set in Amazon SES that should be applied to emails sent via Cognito.
     *
     * @default - no configuration set
     * @see https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-cognito-userpool-emailconfiguration.html#cfn-cognito-userpool-emailconfiguration-configurationset
     * @experimental
     */
    readonly configurationSetName?: string;
    /**
     * (experimental) Required if the UserPool region is different than the SES region.
     *
     * If sending emails with a Amazon SES verified email address,
     * and the region that SES is configured is different than the
     * region in which the UserPool is deployed, you must specify that
     * region here.
     *
     * Must be 'us-east-1', 'us-west-2', or 'eu-west-1'
     *
     * @default - The same region as the Cognito UserPool
     * @experimental
     */
    readonly sesRegion?: string;
}
/**
 * Result of binding email settings with a user pool
 */
interface UserPoolEmailConfig {
    /**
     * The name of the configuration set in SES.
     *
     * @default - none
     */
    readonly configurationSet?: string;
    /**
     * Specifies whether to use Cognito's built in email functionality
     * or SES.
     *
     * @default - Cognito built in email functionality
     */
    readonly emailSendingAccount?: string;
    /**
     * Identifies either the sender's email address or the sender's
     * name with their email address.
     *
     * If emailSendingAccount is DEVELOPER then this cannot be specified.
     *
     * @default 'no-reply@verificationemail.com'
     */
    readonly from?: string;
    /**
     * The destination to which the receiver of the email should reply to.
     *
     * @default - same as `from`
     */
    readonly replyToEmailAddress?: string;
    /**
     * The ARN of a verified email address in Amazon SES.
     *
     * required if emailSendingAccount is DEVELOPER or if
     * 'from' is provided.
     *
     * @default - none
     */
    readonly sourceArn?: string;
}
/**
 * (experimental) Configure how Cognito sends emails.
 *
 * @experimental
 */
export declare abstract class UserPoolEmail {
    /**
     * (experimental) Send email using Cognito.
     *
     * @experimental
     */
    static withCognito(replyTo?: string): UserPoolEmail;
    /**
     * (experimental) Send email using SES.
     *
     * @experimental
     */
    static withSES(options: UserPoolSESOptions): UserPoolEmail;
    /**
     * Returns the email configuration for a Cognito UserPool
     * that controls how Cognito will send emails
     * @internal
     */
    abstract _bind(scope: Construct): UserPoolEmailConfig;
}
export {};
