import { TagType } from './cfn-resource';
import { IResolvable } from './resolvable';
/**
 * (experimental) Interface to implement tags.
 *
 * @experimental
 */
export interface ITaggable {
    /**
     * (experimental) TagManager to set, remove and format tags.
     *
     * @experimental
     */
    readonly tags: TagManager;
}
/**
 * (experimental) Options to configure TagManager behavior.
 *
 * @experimental
 */
export interface TagManagerOptions {
    /**
     * (experimental) The name of the property in CloudFormation for these tags.
     *
     * Normally this is `tags`, but Cognito UserPool uses UserPoolTags
     *
     * @default "tags"
     * @experimental
     */
    readonly tagPropertyName?: string;
}
/**
 * (experimental) TagManager facilitates a common implementation of tagging for Constructs.
 *
 * Normally, you do not need to use this class, as the CloudFormation specification
 * will indicate which resources are taggable. However, sometimes you will need this
 * to make custom resources taggable. Used `tagManager.renderedTags` to obtain a
 * value that will resolve to the tags at synthesis time.
 *
 * @experimental
 * @example
 *
 * import * as cdk from '@aws-cdk/core';
 *
 * class MyConstruct extends cdk.Resource implements cdk.ITaggable {
 * public readonly tags = new cdk.TagManager(cdk.TagType.KEY_VALUE, 'Whatever::The::Type');
 *
 * constructor(scope: cdk.Construct, id: string) {
 * super(scope, id);
 *
 * new cdk.CfnResource(this, 'Resource', {
 * type: 'Whatever::The::Type',
 * properties: {
 * // ...
 * Tags: this.tags.renderedTags,
 * },
 * });
 * }
 * }
 */
export declare class TagManager {
    /**
     * (experimental) Check whether the given construct is Taggable.
     *
     * @experimental
     */
    static isTaggable(construct: any): construct is ITaggable;
    /**
     * (experimental) The property name for tag values.
     *
     * Normally this is `tags` but some resources choose a different name. Cognito
     * UserPool uses UserPoolTags
     *
     * @experimental
     */
    readonly tagPropertyName: string;
    /**
     * (experimental) A lazy value that represents the rendered tags at synthesis time.
     *
     * If you need to make a custom construct taggable, use the value of this
     * property to pass to the `tags` property of the underlying construct.
     *
     * @experimental
     */
    readonly renderedTags: IResolvable;
    private readonly tags;
    private readonly priorities;
    private readonly tagFormatter;
    private readonly resourceTypeName;
    private readonly initialTagPriority;
    /**
     * @experimental
     */
    constructor(tagType: TagType, resourceTypeName: string, tagStructure?: any, options?: TagManagerOptions);
    /**
     * (experimental) Adds the specified tag to the array of tags.
     *
     * @experimental
     */
    setTag(key: string, value: string, priority?: number, applyToLaunchedInstances?: boolean): void;
    /**
     * (experimental) Removes the specified tag from the array if it exists.
     *
     * @param key The tag to remove.
     * @param priority The priority of the remove operation.
     * @experimental
     */
    removeTag(key: string, priority: number): void;
    /**
     * (experimental) Renders tags into the proper format based on TagType.
     *
     * This method will eagerly render the tags currently applied. In
     * most cases, you should be using `tagManager.renderedTags` instead,
     * which will return a `Lazy` value that will resolve to the correct
     * tags at synthesis time.
     *
     * @experimental
     */
    renderTags(): any;
    /**
     * (experimental) Render the tags in a readable format.
     *
     * @experimental
     */
    tagValues(): Record<string, string>;
    /**
     * (experimental) Determine if the aspect applies here.
     *
     * Looks at the include and exclude resourceTypeName arrays to determine if
     * the aspect applies here
     *
     * @experimental
     */
    applyTagAspectHere(include?: string[], exclude?: string[]): boolean;
    /**
     * (experimental) Returns true if there are any tags defined.
     *
     * @experimental
     */
    hasTags(): boolean;
    private _setTag;
    private get sortedTags();
}
